@@ -117,7 +117,7 @@ keepnver(0), maxtempdelay(27), vrangeops(1), dlretriesmax(2);
 int dlbufsize(30000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(40), updinterval(0),
 forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL),
-stucksecs(500), persistoutgoing(1), pipelinelen(10), exsupcount(RESERVED_DEFVAL),
+stucksecs(RESERVED_DEFVAL), persistoutgoing(1), pipelinelen(10), exsupcount(RESERVED_DEFVAL),
 optproxytimeout(-1), patrace(false), maxredlsize(1<<16), nsafriendly(false),
 trackfileuse(false), exstarttradeoff(500000000), fasttimeout(4), discotimeout(15);
 
@@ -135,6 +135,7 @@ int dnscachetime(1800);
 string ACNG_API agentname("Apt-Cacher-NG/" ACVERSION);
 string ACNG_API remoteport("80"), port(ACNG_DEF_PORT);
 string ACNG_API agentheader;
+string ACNG_API dnsresconf("/etc/resolv.conf");
 
 string ACNG_API requestapx;
 string sigbuscmd;
@@ -155,7 +156,7 @@ int conprotos[2] = { PF_UNSPEC, PF_UNSPEC };
 std::atomic_bool degraded(false);
 
 int allocspace = 1024*1024;
-
+int dnsopts = 0;
 }
 
 }
