@@ -186,26 +186,28 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry)
 				}
 			}
 
-			if(entry.fpr.csType != descHave.fpr.csType &&
-					!descHave.fpr.ScanFile(sPathAbs, entry.fpr.csType))
+			if (entry.sFileName != "Release" && entry.sFileName != "InRelease" )
 			{
-				// IO error? better keep it for now, not sure how to deal with it
-				SendFmt << ECLASS "An error occurred while checksumming "
-				<< sPathRel << ", leaving as-is for now.";
-				log::err(tSS() << "Error reading " << sPathAbs );
-				AddDelCbox(sPathRel, "IO error");
-				SendFmt<<CLASSEND;
-				return false;
+				if(entry.fpr.csType != descHave.fpr.csType &&
+						!descHave.fpr.ScanFile(sPathAbs, entry.fpr.csType))
+				{
+					// IO error? better keep it for now, not sure how to deal with it
+					SendFmt << ECLASS "An error occurred while checksumming "
+					<< sPathRel << ", leaving as-is for now.";
+					log::err(tSS() << "Error reading " << sPathAbs );
+					AddDelCbox(sPathRel, "IO error");
+					SendFmt<<CLASSEND;
+					return false;
+				}
+
+				// ok, now fingerprint data must be consistent
+
+				if(!descHave.fpr.csEquals(entry.fpr))
+				{
+					SendFmt << ECLASS << "checksum mismatch on " << sPathRel;
+					return finish_bad("checksum mismatch");
+				}
 			}
-
-			// ok, now fingerprint data must be consistent
-
-			if(!descHave.fpr.csEquals(entry.fpr))
-			{
-				SendFmt << ECLASS << "checksum mismatch on " << sPathRel;
-				return finish_bad("checksum mismatch");
-			}
-
 			// good, or cannot check so must be good
 			if(entry.fpr.size<0 || (descHave.fpr.size == entry.fpr.size))
 				return report_good(lenFromStat);
@@ -673,13 +675,12 @@ void expiration::TrimFiles()
 		{
 //			SendFmt << "let's truncate " << fil << " to " << stinfo.st_size << "<br>";
 			// it's unlikely to be accessed but better protect it
-			fileItemMgmt item;
-			item.PrepareRegisteredFileItemWithStorage(fil, true);
-			if(!item)
+			auto user = TFileItemUser::Create(fil, true);
+			if(!user)
 				continue;
-			lockguard g(item.getFiPtr().get());
+			lockguard g(user.getFiPtr().get());
 			off_t nix;
-			if(item.getFiPtr()->GetStatusUnlocked(nix) >= fileitem::FIST_DLGOTHEAD)
+			if(user.getFiPtr()->GetStatusUnlocked(nix) >= fileitem::FIST_DLGOTHEAD)
 				continue;
 
 			if(0 != truncate(fil.c_str(), stinfo.st_size))
