@@ -54,8 +54,8 @@ fileitem::fileitem() :
 			m_nDlRefsCount(0),
 			usercount(0),
 			m_status(FIST_FRESH),
-			m_nTimeDlStarted(0),
-			m_nTimeDlDone(END_OF_TIME),
+			// good enough to not trigger the makeWay check but also not cause overflows
+			m_nTimeDlStarted(END_OF_TIME-MAXTEMPDELAY*2),
 			m_globRef(mapItems.end())
 {
 }
@@ -335,11 +335,14 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 		bool bForcedRestart, bool &bDoCleanRetry)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
+
+	m_nTimeDlStarted = GetTime();
+
 	auto SETERROR = [&](LPCSTR x) {
 		m_bAllowStoreData=false;
 		m_head.frontLine=mstring("HTTP/1.1 ")+x;
 		m_head.set(header::XORIG, h.h[header::XORIG]);
-		m_status=FIST_DLERROR; m_nTimeDlDone=GetTime();
+		m_status=FIST_DLERROR;
 		_LogWithErrno(x, m_sPathRel);
 	};
 
@@ -362,9 +365,6 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 	if (m_status > FIST_DLPENDING && !bForcedRestart)
 		return false;
 
-	if(m_bCheckFreshness)
-		m_nTimeDlStarted = GetTime();
-
 	m_nIncommingCount+=hDataLen;
 
 	// optional optimization: hints for the filesystem resp. kernel
@@ -393,7 +393,6 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 		::unlink(sHeadPath.c_str());
 
 		m_status=FIST_DLERROR;
-		m_nTimeDlDone=GetTime();
 		return false;
 			};
 
@@ -567,14 +566,41 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 		// every error after that leads to full cleanup to not risk inconsistent file contents 
 
 		int flags = O_WRONLY | O_CREAT | O_BINARY;
+
 		struct stat stbuf;
 
 		mkbasedir(sPathAbs);
+		// 0 might also be a sign of missing metadata while data file may exist
+		// in that case use the other strategy of new file creation which does not crash mmap
+		if (m_nSizeSeen == 0)
+		{
+			checkforceclose(m_filefd); // be sure about that
+			auto tname(sPathAbs + "-"), tname2(sPathAbs + "~");
+			// keep the file descriptor later if needed
+			unique_fd tmp(open(tname.c_str(), flags, cfg::fileperms | O_TRUNC));
+			if (tmp.m_p == -1)
+				return withError("503 Cannot create cache files");
+			fdatasync(tmp.m_p);
+			bool didExist = true;
+			if (0 != rename(sPathAbs.c_str(), tname2.c_str()))
+			{
+				if (errno != ENOENT)
+					return withError("503 Cannot move cache files");
+				didExist = false;
+			}
+			if (0 != rename(tname.c_str(), sPathAbs.c_str()))
+				return withError("503 Cannot rename cache files");
+			if (!didExist)
+				unlink(tname2.c_str());
+			std::swap(m_filefd, tmp.m_p);
+		}
+		else
+			m_filefd = open(sPathAbs.c_str(), flags, cfg::fileperms);
 
-		m_filefd = open(sPathAbs.c_str(), flags, cfg::fileperms);
 		ldbg("file opened?! returned: " << m_filefd);
 
 		// self-recovery from cache poisoned with files with wrong permissions
+		// we still want to recover the file content if we can
 		if (m_filefd < 0)
 		{
 			if(m_nSizeChecked > 0) // OOOH CRAP! CANNOT APPEND HERE! Do what's still possible.
@@ -647,18 +673,22 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, size_t
 #endif
 		 */
 		ldbg("Storing header as " + sHeadPath);
-		int count=m_head.StoreToFile(sHeadPath);
+		auto tempHead(sPathAbs + ".hea%");
+		int count = m_head.StoreToFile(tempHead);
 
 		if(count<0)
-			return withErrorAndKillFile( (-count != ENOSPC
-					? "503 Cache storage error" : "503 OUT OF DISK SPACE"));
+		{
+			errno = -count;
+			return withError(tErrnoFmter("503 Cache error, ").c_str());
+		}
 
-		// double-check the sane state
-		if(0 != fstat(m_filefd, &stbuf) || stbuf.st_size != m_nSizeChecked)
-			return withErrorAndKillFile("503 Inconsistent file state");
+		// by now the file should have been validated and adjusted to the correct size
+		if (m_nSizeChecked != lseek(m_filefd, 0, SEEK_END))
+			return withError("503 Cache error, size trimming failed");
 
-		if(m_nSizeChecked != lseek(m_filefd, m_nSizeChecked, SEEK_SET))
-			return withErrorAndKillFile("503 IO error, positioning");
+		// this is not supposed to go wrong at this moment
+		if (0 != rename(tempHead.c_str(), sHeadPath.c_str()))
+			return withError("503 Cache error, renaming failure");
 	}
 
 	m_status=FIST_DLGOTHEAD;
@@ -690,7 +720,6 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 		else
 		{
 			m_status = FIST_COMPLETE;
-			m_nTimeDlDone = GetTime();
 
 			if (cfg::debug & log::LOG_MORE)
 				log::misc(tSS() << "Download of " << m_sPathRel << " finished");
@@ -715,7 +744,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 			while(size > 0)
 			{
 				int r = write(m_filefd, data, size);
-				if(r < 0)
+				if (r < 0)
 				{
 					if(EINTR == errno || EAGAIN == errno)
 						continue;
@@ -723,7 +752,6 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 					m_head.frontLine = efmt;
 					m_status=FIST_DLERROR;
 					// message will be set by the caller
-					m_nTimeDlDone = GetTime();
 					_LogWithErrno(efmt.c_str(), m_sPathRel);
 					return false;
 				}
@@ -809,22 +837,24 @@ TFileItemUser TFileItemUser::Create(cmstring &sPathUnescaped, bool makeWay)
 			{
 				// detect items that got stuck somehow and move it out of the way
 				time_t now(GetTime());
-				if(fi->m_nTimeDlDone > now + MAXTEMPDELAY  + 2
-						|| fi->m_nTimeDlStarted > now - cfg::stucksecs)
+				if(now > (fi->m_nTimeDlStarted + cfg::stucksecs))
 				{
+					auto altPathRel = fi->m_sPathRel + "." + ltos(now);
+					auto altPathAbs = SABSPATH(altPathRel);
 
 					auto pathAbs = SABSPATH(fi->m_sPathRel);
-					auto xName = pathAbs + ltos(now);
-					if(0 != creat(xName.c_str(), cfg::fileperms))
+
+					if (0 != link(pathAbs.c_str(), altPathAbs.c_str())
+							|| 0 != unlink(pathAbs.c_str()))
 					{
 						// oh, that's bad, no permissions on the folder whatsoever?
-						log::err(string("Failure to create replacement of ") + fi->m_sPathRel + " - CHECK FOLDER PERMISSIONS!");
+						log::err(string("Failed to move stale item ") + fi->m_sPathRel + " out of the way: " + tErrnoFmter());
 					}
 					else
 					{
-						// if we can create files there then renaming should not be a problem
-						unlink(pathAbs.c_str());
-						rename(xName.c_str(), pathAbs.c_str());
+						fi->m_sPathRel = altPathRel;
+						fi->m_bIsGarbage = true;
+
 						fi->m_globRef = mapItems.end();
 						mapItems.erase(it);
 						goto add_as_new;
@@ -983,7 +1013,10 @@ void TFileItemUser::dump_status()
 
 fileitem_with_storage::~fileitem_with_storage()
 {
-	Truncate2checkedSize();
+	if (m_bIsGarbage && !m_sPathRel.empty())
+		unlink(SZABSPATH(m_sPathRel));
+	else
+		Truncate2checkedSize();
 }
 
 int fileitem_with_storage::Truncate2checkedSize()
